@@ -168,6 +168,8 @@ struct _CellEdit {
 
 	guint pointer_in : 1;
 	guint default_cursor_shown : 1;
+
+	ECellActions actions;
 };
 
 static void e_cell_text_view_command (ETextEventProcessor *tep, ETextEventProcessorCommand *command, gpointer data);
@@ -481,7 +483,7 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
 
 	cursor_color = foreground;
 
-	if (ect->color_column != -1) {
+	if (ect->color_column != -1 && ! selected) {
 		color_spec = e_table_model_value_at (ecell_view->e_table_model,
 						     ect->color_column, row);
 		cell_foreground = e_cell_text_get_color (text_view,
@@ -871,7 +873,7 @@ ect_edit_select_all (ECellTextView *text_view)
  * ECell::event method
  */
 static gint
-ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col, int row, ECellFlags flags)
+ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col, int row, ECellFlags flags, ECellActions *actions)
 {
 	ECellTextView *text_view = (ECellTextView *) ecell_view;
 	ETextEventProcessorEvent e_tep_event;
@@ -943,6 +945,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 
 				_get_tep (edit);
 				return_val = e_text_event_processor_handle_event (edit->tep, &e_tep_event);
+				*actions = edit->actions;
 				if (e_tep_event.key.string) g_free (e_tep_event.key.string);
 				break;
 			}
@@ -974,6 +977,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			_get_tep (edit);
 			return_val = e_text_event_processor_handle_event (edit->tep,
 									  &e_tep_event);
+			*actions = edit->actions;
 			if (event->button.button == 1) {
 				if (event->type == GDK_BUTTON_PRESS)
 					edit->button_down = TRUE;
@@ -995,6 +999,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			_get_tep (edit);
 			return_val = e_text_event_processor_handle_event (edit->tep,
 									  &e_tep_event);
+			*actions = edit->actions;
 			if (event->button.button == 1) {
 				if (event->type == GDK_BUTTON_PRESS)
 					edit->button_down = TRUE;
@@ -1017,6 +1022,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			_get_tep (edit);
 			return_val = e_text_event_processor_handle_event (edit->tep,
 									  &e_tep_event);
+			*actions = edit->actions;
 			edit->lastx = motion.x;
 			edit->lasty = motion.y;
 			edit->last_state = motion.state;
@@ -1342,6 +1348,7 @@ ect_show_tooltip (ECellView *ecell_view,
 		  int model_col,
 		  int view_col,
 		  int row,
+		  int col_width,
 		  ETableTooltip *tooltip)
 {
 	ECellTextView *text_view = (ECellTextView *) ecell_view;
@@ -1366,6 +1373,10 @@ ect_show_tooltip (ECellView *ecell_view,
 	tooltip->timer = 0;
 
 	build_current_cell (&cell, text_view, model_col, view_col, row);
+
+	set_style(ecell_view, &cell, row);
+
+	cell.width = col_width - 8;
 	split_into_lines (&cell);
 	calc_line_widths (&cell);
 
@@ -1405,7 +1416,7 @@ ect_show_tooltip (ECellView *ecell_view,
 		gdouble line_width;
 
 		line_width = e_font_utf8_text_width (text_view->font, 
-						     E_FONT_PLAIN, lines->text,
+						     cell.style, lines->text,
 						     lines->length);
 		max_width = MAX (max_width, line_width);
 	}
@@ -1417,7 +1428,7 @@ ect_show_tooltip (ECellView *ecell_view,
 				      "y1", (double) 0.0,
 				      "x2", (double) max_width + 4,
 				      "y2", (double) text_height,
-				      "fill_color", "yellow",
+				      "fill_color", "light gray",
 				      NULL);
 
 	tooltip_text = gnome_canvas_item_new (gnome_canvas_root (GNOME_CANVAS (canvas)),
@@ -1797,11 +1808,10 @@ _get_position_from_xy (CurrentCell *cell, gint x, gint y)
 static gboolean
 _blink_scroll_timeout (gpointer data)
 {
-	CurrentCell *cell = CURRENT_CELL(data);
 	ECellTextView *text_view = (ECellTextView *) data;
 	ECellText *ect = E_CELL_TEXT (((ECellView *)text_view)->ecell);
 	CellEdit *edit = text_view->edit;
-	ECellTextLineBreaks *linebreaks = cell->breaks;
+	CurrentCell *cell = CURRENT_CELL(edit);
 
 	gulong current_time;
 	gboolean scroll = FALSE;
@@ -1820,12 +1830,17 @@ _blink_scroll_timeout (gpointer data)
 	}
 	if (scroll && edit->button_down) {
 		/* FIXME: Copy this for y. */
-		if (edit->lastx - ect->x > cell->width &&
-		    edit->xofs_edit < linebreaks->max_width - cell->width) {
-			edit->xofs_edit += 4;
-			if (edit->xofs_edit > linebreaks->max_width - cell->width + 1)
-				edit->xofs_edit = linebreaks->max_width - cell->width + 1;
-			redraw = TRUE;
+		if (edit->lastx - ect->x > cell->width) {
+			ECellTextLineBreaks *linebreaks;
+			split_into_lines (cell);
+			linebreaks = cell->breaks;
+			if (edit->xofs_edit < linebreaks->max_width - cell->width) {
+				edit->xofs_edit += 4;
+				if (edit->xofs_edit > linebreaks->max_width - cell->width + 1)
+					edit->xofs_edit = linebreaks->max_width - cell->width + 1;
+				redraw = TRUE;
+			}
+			unref_lines (cell);
 		}
 		if (edit->lastx - ect->x < 0 &&
 		    edit->xofs_edit > 0) {
@@ -2139,17 +2154,11 @@ e_cell_text_view_command (ETextEventProcessor *tep, ETextEventProcessorCommand *
 		edit->select_by_word = command->value;
 		break;
 	case E_TEP_GRAB:
+		edit->actions = E_CELL_GRAB;
+		break;
 	case E_TEP_UNGRAB:
-#if 0
-	case E_TEP_GRAB:
-		gnome_canvas_item_grab (GNOME_CANVAS_ITEM(text), 
-					GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK,
-					text_view->i_cursor,
-					command->time);
+		edit->actions = E_CELL_UNGRAB;
 		break;
-		gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM(text), command->time);
-		break;
-#endif
 	case E_TEP_NOP:
 		break;
 	}
@@ -2378,7 +2387,7 @@ split_into_lines (CurrentCell *cell)
 	gint len;
 
 	char *text = cell->text;
-	ECellTextLineBreaks *linebreaks = cell->breaks;
+	ECellTextLineBreaks *linebreaks;
 
 	if (! cell->breaks) {
 		cell->breaks = g_new (ECellTextLineBreaks, 1);
@@ -2444,7 +2453,7 @@ calc_ellipsis (ECellTextView *text_view)
 	EFont *font;
 	
 	font = text_view->font;
-	if (font)
+	if (font) {
 		text_view->ellipsis_width[E_FONT_PLAIN] =
 			e_font_utf8_text_width (font, E_FONT_PLAIN,
 					ect->ellipsis ? ect->ellipsis : "...",
@@ -2453,6 +2462,7 @@ calc_ellipsis (ECellTextView *text_view)
 			e_font_utf8_text_width (font, E_FONT_BOLD,
 					ect->ellipsis ? ect->ellipsis : "...",
 					ect->ellipsis ? strlen (ect->ellipsis) : 3);
+	}
 }
 
 /* Calculates the line widths (in pixels) of the text's splitted lines */
@@ -2493,8 +2503,9 @@ calc_line_widths (CurrentCell *cell)
 					lines->ellipsis_length = 0;
 					for (j = 0; j < lines->length; j++){
 						if (e_font_utf8_text_width (font, cell->style, lines->text, j) +
-						    text_view->ellipsis_width[cell->style] <= cell->width)
+						    text_view->ellipsis_width[cell->style] < cell->width) {
 							lines->ellipsis_length = j;
+						}
 						else
 							break;
 					}
