@@ -300,7 +300,7 @@ ect_draw (ECellView *ecell_view, GdkDrawable *drawable,
  * ECell::event method
  */
 static gint
-ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col, int row, ECellFlags flags)
+ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col, int row, ECellFlags flags, ECellActions *actions)
 {
 	ECellTreeView *tree_view = (ECellTreeView *) ecell_view;
 	ETreeModel *tree_model = e_cell_tree_get_tree_model (ecell_view->e_table_model, row);
@@ -318,13 +318,15 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 				e_tree_model_node_set_expanded (tree_model,
 								node,
 								!e_tree_model_node_is_expanded(tree_model, node));
+				return TRUE;
 			}
-			return TRUE;
 		}
 		else if (event->button.x < (offset - INDENT_AMOUNT))
 			return FALSE;
 	}
-	default:
+	default: {
+		gint return_value;
+
 		/* modify the event and pass it off to our subcell_view */
 		switch (event->type) {
 		case GDK_BUTTON_PRESS:
@@ -339,7 +341,26 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 		default:
 			/* nada */
 		}
-		return e_cell_event(tree_view->subcell_view, event, model_col, view_col, row, flags);
+
+		return_value = e_cell_event(tree_view->subcell_view, event, model_col, view_col, row, flags, actions);
+
+		/* modify the event and pass it off to our subcell_view */
+		switch (event->type) {
+		case GDK_BUTTON_PRESS:
+		case GDK_BUTTON_RELEASE:
+		case GDK_2BUTTON_PRESS:
+		case GDK_3BUTTON_PRESS:
+			event->button.x += offset;
+			break;
+		case GDK_MOTION_NOTIFY:
+			event->motion.x += offset;
+			break;
+		default:
+			/* nada */
+		}
+
+		return return_value;
+	}
 	}
 }
 
@@ -417,31 +438,20 @@ ect_max_width (ECellView *ecell_view, int model_col, int view_col)
  */
 static void
 ect_show_tooltip (ECellView *ecell_view, int model_col, int view_col, int row,
-		  ETableTooltip *tooltip)
+		  int col_width, ETableTooltip *tooltip)
 {		
 	ECellTreeView *tree_view = (ECellTreeView *) ecell_view;
 	ETreeModel *tree_model = e_cell_tree_get_tree_model (ecell_view->e_table_model, row);
 	ETreePath *node = e_cell_tree_get_node (ecell_view->e_table_model, row);
 	int offset = offset_of_node (tree_model, node);
-
-	{
-		tooltip->x += offset;
-		e_cell_show_tooltip(tree_view->subcell_view, model_col, view_col, row, tooltip);
-	}
-#if 0
 	GdkPixbuf *node_image;
 
 	node_image = e_tree_model_icon_of_node (tree_model, node);
 	if (node_image)
 		offset += gdk_pixbuf_get_width (node_image);
 
-	/* if the tooltip happened in the subcell, then handle it */
-	
-	if (tooltip->cx > offset) {
-		tooltip->x += offset;
-		e_cell_show_tooltip (tree_view->subcell_view, model_col, view_col, row, tooltip);
-	}
-#endif
+	tooltip->x += offset;
+	e_cell_show_tooltip (tree_view->subcell_view, model_col, view_col, row, col_width - offset, tooltip);
 }
 		
 /*
