@@ -180,29 +180,6 @@ e_table_group_set_focus (ETableGroup *etg,
 		ETG_CLASS (etg)->set_focus (etg, direction, row);
 }
 
-void
-e_table_group_set_cursor_row (ETableGroup *etg,
-			      gint row)
-{
-	g_return_if_fail (etg != NULL);
-	g_return_if_fail (E_IS_TABLE_GROUP (etg));
-
-	if (ETG_CLASS (etg)->set_cursor_row)
-		ETG_CLASS (etg)->set_cursor_row (etg, row);
-}
-
-int
-e_table_group_get_cursor_row (ETableGroup *etg)
-{
-	g_return_val_if_fail (etg != NULL, -1);
-	g_return_val_if_fail (E_IS_TABLE_GROUP (etg), -1);
-
-	if (ETG_CLASS (etg)->get_cursor_row)
-		return ETG_CLASS (etg)->get_cursor_row (etg);
-	else
-		return -1;
-}
-
 gboolean
 e_table_group_get_focus (ETableGroup *etg)
 {
@@ -273,14 +250,14 @@ e_table_group_cursor_change (ETableGroup *e_table_group, gint row)
 }
 
 void
-e_table_group_double_click (ETableGroup *e_table_group, gint row)
+e_table_group_double_click (ETableGroup *e_table_group, gint row, gint col, GdkEvent *event)
 {
 	g_return_if_fail (e_table_group != NULL);
 	g_return_if_fail (E_IS_TABLE_GROUP (e_table_group));
 
 	gtk_signal_emit (GTK_OBJECT (e_table_group),
 			 etg_signals [DOUBLE_CLICK],
-			 row);
+			 row, col, event);
 }
 
 gint
@@ -390,8 +367,6 @@ etg_class_init (GtkObjectClass *object_class)
 	klass->increment  = NULL;
 	klass->decrement  = NULL;
 	klass->set_focus  = NULL;
-	klass->set_cursor_row = NULL;
-	klass->get_cursor_row = NULL;
 	klass->get_focus = etg_get_focus;
 	klass->get_ecol = NULL;
 	klass->get_printable = NULL;
@@ -412,8 +387,8 @@ etg_class_init (GtkObjectClass *object_class)
 				GTK_RUN_LAST,
 				object_class->type,
 				GTK_SIGNAL_OFFSET (ETableGroupClass, double_click),
-				gtk_marshal_NONE__INT,
-				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+				gtk_marshal_NONE__INT_INT_POINTER,
+				GTK_TYPE_NONE, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_POINTER);
 
 	etg_signals [RIGHT_CLICK] =
 		gtk_signal_new ("right_click",
