@@ -1016,36 +1016,39 @@ eti_init (GnomeCanvasItem *item)
 {
 	ETableItem *eti = E_TABLE_ITEM (item);
 
-	eti->editing_col = -1;
-	eti->editing_row = -1;
-	eti->height = 0;
-	eti->width = 0;
-	eti->minimum_width = 0;
+	eti->editing_col               = -1;
+	eti->editing_row               = -1;
+	eti->height                    = 0;
+	eti->width                     = 0;
+	eti->minimum_width             = 0;
 
-	eti->height_cache = NULL;
-	eti->height_cache_idle_id = 0;
-	eti->height_cache_idle_count = 0;
+	eti->height_cache              = NULL;
+	eti->height_cache_idle_id      = 0;
+	eti->height_cache_idle_count   = 0;
 	
-	eti->length_threshold = -1;
+	eti->length_threshold          = -1;
 	eti->renderers_can_change_size = 1;
 
-	eti->uses_source_model = 0;
-	eti->source_model = NULL;
+	eti->uses_source_model         = 0;
+	eti->source_model              = NULL;
 	
-	eti->row_guess = -1;
-	eti->cursor_mode = E_TABLE_CURSOR_SIMPLE;
+	eti->row_guess                 = -1;
+	eti->cursor_mode               = E_TABLE_CURSOR_SIMPLE;
 
-	eti->selection_change_id = 0;
-	eti->cursor_change_id = 0;
-	eti->selection = NULL;
+	eti->selection_change_id       = 0;
+	eti->cursor_change_id          = 0;
+	eti->selection                 = NULL;
 
-	eti->needs_redraw = 0;
-	eti->needs_compute_height = 0;
+	eti->needs_redraw              = 0;
+	eti->needs_compute_height      = 0;
 
-	eti->tooltip = g_new0 (ETableTooltip, 1);
-	eti->tooltip->timer = 0;
-	eti->tooltip->window = NULL;
-	eti->tooltip->eti = GNOME_CANVAS_ITEM (eti);
+	eti->tooltip                   = g_new0 (ETableTooltip, 1);
+	eti->tooltip->timer            = 0;
+	eti->tooltip->window           = NULL;
+	eti->tooltip->eti              = GNOME_CANVAS_ITEM (eti);
+
+	eti->grabbed_col               = -1;
+	eti->grabbed_row               = -1;
 
 	e_canvas_item_set_reflow_callback (GNOME_CANVAS_ITEM (eti), eti_reflow);
 }
@@ -1055,12 +1058,30 @@ eti_init (GnomeCanvasItem *item)
 static const char gray50_bits[] = {
 	0x02, 0x01, };
 
+#if 0
+static gint
+eti_visibility_notify(GtkWidget	         *widget,
+		      GdkEventVisibility *event,
+		      ETableItem         *eti)
+{
+	if (eti->tooltip->window) {
+		gtk_widget_destroy (eti->tooltip->window);
+		eti->tooltip->window = NULL;
+	}
+
+	return FALSE;
+}
+#endif
+
 static void
 eti_realize (GnomeCanvasItem *item)
 {
 	ETableItem *eti = E_TABLE_ITEM (item);
 	GtkWidget *canvas_widget = GTK_WIDGET (item->canvas);
 	GdkWindow *window;
+#if 0
+	GdkEventMask mask;
+#endif
 	
 	if (GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->realize)
                 (*GNOME_CANVAS_ITEM_CLASS (eti_parent_class)->realize)(item);
@@ -1087,6 +1108,15 @@ eti_realize (GnomeCanvasItem *item)
 	gdk_gc_set_stipple (eti->focus_gc, eti->stipple);
 	gdk_gc_set_fill (eti->focus_gc, GDK_OPAQUE_STIPPLED);
 
+#if 0
+	mask = gtk_widget_get_events(GTK_WIDGET(item->canvas));
+	mask |= GDK_VISIBILITY_NOTIFY_MASK;
+	gtk_widget_set_events(GTK_WIDGET(item->canvas), mask);
+
+	gtk_signal_connect(GTK_OBJECT(item->canvas), "visibility_notify_event",
+			   GTK_SIGNAL_FUNC(eti_visibility_notify), eti);
+#endif
+
 	if (eti->cell_views == NULL)
 		eti_attach_cell_views (eti);
 	
@@ -1362,6 +1392,14 @@ find_cell (ETableItem *eti, double x, double y, int *col_res, int *row_res, doub
 	int col, row;
 	
 	/* FIXME: this routine is inneficient, fix later */
+
+	if (eti->grabbed_col >= 0 && eti->grabbed_row >= 0) {
+		*col_res = eti->grabbed_col;
+		*row_res = eti->grabbed_row;
+		*x1_res = x - eti->x1 - e_table_header_col_diff (eti->header, 0, eti->grabbed_col);
+		*y1_res = y - eti->y1 - eti_row_diff (eti, 0, eti->grabbed_row);
+		return TRUE;
+	}
 	
 	if (cols == 0 || rows == 0)
 		return FALSE;
@@ -1463,6 +1501,11 @@ _do_tooltip (ETableItem *eti)
 	int x = 0, y = 0;
 	int i;
 
+	if (eti->tooltip->window) {
+		gtk_widget_destroy (eti->tooltip->window);
+		eti->tooltip->window = NULL;
+	}
+
 	if (eti_editing (eti))
 		return FALSE;
 
@@ -1481,10 +1524,34 @@ _do_tooltip (ETableItem *eti)
 			     view_to_model_col (eti, eti->tooltip->col),
 			     eti->tooltip->col,
 			     eti->tooltip->row,
+			     eti->header->columns[eti->tooltip->col]->width,
 			     eti->tooltip);
 	return FALSE;
 }
 
+static gint
+eti_e_cell_event     (ETableItem *item, ECellView *ecell_view, GdkEvent *event, int time, int model_col, int view_col, int row, ECellFlags flags)
+{
+	ECellActions actions = 0;
+	gint ret_val;
+
+	ret_val = e_cell_event (ecell_view, event, model_col, view_col, row, flags, &actions);
+
+	if (actions & E_CELL_GRAB) {
+		gnome_canvas_item_grab(GNOME_CANVAS_ITEM(item), GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK, NULL, time);
+		item->grabbed_col = view_col;
+		item->grabbed_row = row;
+	}
+
+	if (actions & E_CELL_UNGRAB) {
+		gnome_canvas_item_ungrab(GNOME_CANVAS_ITEM(item), time);
+		item->grabbed_col = -1;
+		item->grabbed_row = -1;
+	}
+
+	return ret_val;
+}
+
 /* FIXME: cursor */
 static int
 eti_event (GnomeCanvasItem *item, GdkEvent *e)
@@ -1519,7 +1586,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			button.x = x1;
 			button.y = y1;
 
-			return_val = e_cell_event (ecell_view, (GdkEvent *) &button, view_to_model_col(eti, col), col, row, 0);
+			return_val = eti_e_cell_event (eti, ecell_view, (GdkEvent *) &button, button.time, view_to_model_col(eti, col), col, row, 0);
 			if (return_val)
 				return TRUE;
 
@@ -1546,8 +1613,8 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				 * Adjust the event positions
 				 */
 
-				return_val = e_cell_event (ecell_view, (GdkEvent *) &button,
-							   view_to_model_col(eti, col), col, row, E_CELL_EDITING);
+				return_val = eti_e_cell_event (eti, ecell_view, (GdkEvent *) &button, button.time, 
+							       view_to_model_col(eti, col), col, row, E_CELL_EDITING);
 			}
 
 			break;
@@ -1606,8 +1673,8 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				e->button.x = x1;
 				e->button.y = y1;
 
-				return_val = e_cell_event (ecell_view, e,
-							   view_to_model_col(eti, col), col, row, E_CELL_EDITING);
+				return_val = eti_e_cell_event (eti, ecell_view, e, e->button.time,
+							       view_to_model_col(eti, col), col, row, E_CELL_EDITING);
 			}
 			break;
 		case 3:
@@ -1623,6 +1690,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 	case GDK_2BUTTON_PRESS: {
 		double x1, y1;
 		int col, row;
+		GdkEventButton button;
 
 		if (e->button.button == 5 ||
 		    e->button.button == 4)
@@ -1633,8 +1701,12 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		if (!find_cell (eti, e->button.x, e->button.y, &col, &row, &x1, &y1))
 			return TRUE;
 
+		button = *(GdkEventButton *)e;
+		button.x = x1;
+		button.y = y1;
+
 		gtk_signal_emit (GTK_OBJECT (eti), eti_signals [DOUBLE_CLICK],
-				 row);
+				 row, view_to_model_col (eti, col), &button);
 		break;
 	}
 	case GDK_MOTION_NOTIFY: {
@@ -1652,13 +1724,18 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			       "cursor_col", &cursor_col,
 			       NULL);
 
+		if (eti->tooltip->window) {
+			gtk_widget_destroy (eti->tooltip->window);
+			eti->tooltip->window = NULL;
+		}
+
 		if (eti->tooltip->timer > 0)
 			gtk_timeout_remove (eti->tooltip->timer);
 		eti->tooltip->col = col;
 		eti->tooltip->row = row;
 		eti->tooltip->cx = e->motion.x;
 		eti->tooltip->cy = e->motion.y;
-		eti->tooltip->timer = gtk_timeout_add (1000, (GSourceFunc)_do_tooltip, eti);
+		eti->tooltip->timer = gtk_timeout_add (100, (GSourceFunc)_do_tooltip, eti);
 
 		if (cursor_row == view_to_model_row(eti, row) && cursor_col == view_to_model_col(eti, col)){
 			ecell_view = eti->cell_views [col];
@@ -1669,8 +1746,8 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			e->motion.x = x1;
 			e->motion.y = y1;
 			
-			return_val = e_cell_event (ecell_view, e,
-						   view_to_model_col(eti, col), col, row, E_CELL_EDITING);
+			return_val = eti_e_cell_event (eti, ecell_view, e, e->motion.time,
+						       view_to_model_col(eti, col), col, row, E_CELL_EDITING);
 		}
 		break;
 	}
@@ -1683,6 +1760,11 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 			       "cursor_col", &cursor_col,
 			       NULL);
 
+		if (eti->tooltip->window) {
+			gtk_widget_destroy (eti->tooltip->window);
+			eti->tooltip->window = NULL;
+		}
+
 		if (cursor_col == -1)
 			return FALSE;
 
@@ -1724,6 +1806,10 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 		case GDK_Tab:
 		case GDK_KP_Tab:
 		case GDK_ISO_Left_Tab:
+			/* Let tab send you to the next widget. */
+			return_val = FALSE;
+			break;
+#if 0
 			if ((e->key.state & GDK_SHIFT_MASK) != 0){
 				/* shift tab */
 				if (cursor_col != view_to_model_col(eti, 0))
@@ -1750,6 +1836,7 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				e_table_item_enter_edit (eti, model_to_view_col(eti, cursor_col), model_to_view_row(eti, cursor_row));
 			}
 			break;
+#endif
 
 		case GDK_Return:
 		case GDK_KP_Enter:
@@ -1759,9 +1846,9 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 				e_table_item_leave_edit (eti);
 #if 0
 				ecell_view = eti->cell_views [eti->editing_col];
-				return_val = e_cell_event (ecell_view, e,
-							   view_to_model_col(eti, eti->editing_col),
-							   eti->editing_col, eti->editing_row, E_CELL_EDITING);
+				return_val = eti_e_cell_event (eti, ecell_view, e, e->key.time,
+							       view_to_model_col(eti, eti->editing_col),
+							       eti->editing_col, eti->editing_row, E_CELL_EDITING);
 #endif
 			}
 			return_val = FALSE;
@@ -1787,9 +1874,9 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 						 model_to_view_row(eti, cursor_row), cursor_col, e, &return_val);
 			} else {
 				ecell_view = eti->cell_views [eti->editing_col];
-				return_val = e_cell_event (ecell_view, e,
-							   view_to_model_col(eti, eti->editing_col),
-							   eti->editing_col, eti->editing_row, E_CELL_EDITING);
+				return_val = eti_e_cell_event (eti, ecell_view, e, e->key.time,
+							       view_to_model_col(eti, eti->editing_col),
+							       eti->editing_col, eti->editing_row, E_CELL_EDITING);
 			}
 		}
 		break;
@@ -1808,9 +1895,9 @@ eti_event (GnomeCanvasItem *item, GdkEvent *e)
 
 		if (eti_editing (eti)){
 			ecell_view = eti->cell_views [eti->editing_col];
-			return_val = e_cell_event (ecell_view, e,
-						   view_to_model_col(eti, eti->editing_col),
-						   eti->editing_col, eti->editing_row, E_CELL_EDITING);
+			return_val = eti_e_cell_event (eti, ecell_view, e, e->key.time,
+						       view_to_model_col(eti, eti->editing_col),
+						       eti->editing_col, eti->editing_row, E_CELL_EDITING);
 		}
 		break;
 	}
@@ -1890,8 +1977,8 @@ eti_class_init (GtkObjectClass *object_class)
 				GTK_RUN_LAST,
 				object_class->type,
 				GTK_SIGNAL_OFFSET (ETableItemClass, double_click),
-				gtk_marshal_NONE__INT,
-				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+				gtk_marshal_NONE__INT_INT_POINTER,
+				GTK_TYPE_NONE, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_POINTER);
 
 	eti_signals [RIGHT_CLICK] =
 		gtk_signal_new ("right_click",
@@ -2070,10 +2157,16 @@ e_table_item_compute_location (ETableItem        *eti,
 			       int               *row,
 			       int               *col)
 {
+	/* Save the grabbed row but make sure that we don't get flawed
+           results because the cursor is grabbed. */
+	int grabbed_row = eti->grabbed_row;
+	eti->grabbed_row = -1;
+
 	if (!find_cell (eti, *x, *y, col, row, NULL, NULL)) {
 		*y -= eti_get_height(eti);
 	}
-	
+
+	eti->grabbed_row = grabbed_row;
 }
 
 typedef struct {
