@@ -24,6 +24,7 @@
 #include "gal/widgets/e-canvas-vbox.h"
 #include "e-table.h"
 #include "e-table-header-item.h"
+#include "e-table-header-utils.h"
 #include "e-table-subset.h"
 #include "e-table-item.h"
 #include "e-table-group.h"
@@ -337,11 +338,11 @@ group_cursor_change (ETableGroup *etg, int row, ETable *et)
 }
 
 static void
-group_double_click (ETableGroup *etg, int row, ETable *et)
+group_double_click (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *et)
 {
 	gtk_signal_emit (GTK_OBJECT (et),
 			 et_signals [DOUBLE_CLICK],
-			 row);
+			 row, col, event);
 }
 
 static gint
@@ -708,6 +709,10 @@ et_state_to_header (ETable *e_table, ETableHeader *full_header, ETableState *sta
 	
 	nh = e_table_header_new ();
 
+	gtk_object_set(GTK_OBJECT(nh),
+		       "width_extras", e_table_header_width_extras(GTK_WIDGET(e_table)->style),
+		       NULL);
+
 	for (column = 0; column < state->col_count; column++) {
 		int col;
 
@@ -747,6 +752,10 @@ e_table_set_state_object(ETable *e_table, ETableState *state)
 					    e_table);
 	}
 
+	if (e_table->sorter)
+		gtk_object_set(GTK_OBJECT(e_table->sorter),
+			       "sort_info", e_table->sort_info,
+			       NULL);
 	if (e_table->header_item)
 		gtk_object_set(GTK_OBJECT(e_table->header_item),
 			       "ETableHeader", e_table->header,
@@ -777,7 +786,7 @@ void            e_table_set_state                 (ETable               *e_table
 	if (state->col_count > 0)
 		e_table_set_state_object(e_table, state);
 
-	gtk_object_sink(GTK_OBJECT(state));
+	gtk_object_unref(GTK_OBJECT(state));
 }
 
 void            e_table_load_state                (ETable               *e_table,
@@ -795,7 +804,7 @@ void            e_table_load_state                (ETable               *e_table
 	if (state->col_count > 0)
 		e_table_set_state_object(e_table, state);
 
-	gtk_object_sink(GTK_OBJECT(state));
+	gtk_object_unref(GTK_OBJECT(state));
 }
 
 ETableState *
@@ -834,7 +843,7 @@ gchar          *e_table_get_state                 (ETable               *e_table
 
 	state = e_table_get_state_object(e_table);
 	string = e_table_state_save_to_string(state);
-	gtk_object_sink(GTK_OBJECT(state));
+	gtk_object_unref(GTK_OBJECT(state));
 	return string;
 }
 
@@ -845,7 +854,7 @@ void            e_table_save_state                (ETable               *e_table
 
 	state = e_table_get_state_object(e_table);
 	e_table_state_save_to_file(state, filename);
-	gtk_object_sink(GTK_OBJECT(state));
+	gtk_object_unref(GTK_OBJECT(state));
 }
 
 static void
@@ -1194,9 +1203,9 @@ e_table_set_cursor_row (ETable *e_table, int row)
 	g_return_if_fail(E_IS_TABLE(e_table));
 	g_return_if_fail(row >= 0);
 
-	row = e_table_sorter_model_to_sorted(e_table->sorter, row);
-	if (row != -1)
-		e_table_group_set_cursor_row(e_table->group, row);
+	gtk_object_set(GTK_OBJECT(e_table->selection),
+		       "cursor_row", row,
+		       NULL);
 }
 
 int
@@ -1206,8 +1215,9 @@ e_table_get_cursor_row (ETable *e_table)
 	g_return_val_if_fail(e_table != NULL, -1);
 	g_return_val_if_fail(E_IS_TABLE(e_table), -1);
 
-	row = e_table_group_get_cursor_row(e_table->group);
-	row = e_table_sorter_sorted_to_model(e_table->sorter, row);
+	gtk_object_get(GTK_OBJECT(e_table->selection),
+		       "cursor_row", &row,
+		       NULL);
 	return row;
 }
 
@@ -1958,8 +1968,8 @@ e_table_class_init (GtkObjectClass *object_class)
 				GTK_RUN_LAST,
 				object_class->type,
 				GTK_SIGNAL_OFFSET (ETableClass, double_click),
-				gtk_marshal_NONE__INT,
-				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
+				gtk_marshal_NONE__INT_INT_POINTER,
+				GTK_TYPE_NONE, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_POINTER);
 
 	et_signals [RIGHT_CLICK] =
 		gtk_signal_new ("right_click",
