@@ -102,7 +102,7 @@ typedef struct {
 
 	void        (*cursor_change)      (ETable *et, int row);
 	void        (*selection_change)   (ETable *et);
-	void        (*double_click)       (ETable *et, int row);
+	void        (*double_click)       (ETable *et, int row, int col, GdkEvent *event);
 	gint        (*right_click)        (ETable *et, int row, int col, GdkEvent *event);
 	gint        (*click)              (ETable *et, int row, int col, GdkEvent *event);
 	gint        (*key_press)          (ETable *et, int row, int col, GdkEvent *event);
