@@ -37,7 +37,7 @@ typedef struct {
 
 GtkType    color_combo_get_type (void);
 
-GtkWidget *color_combo_new      (char **icon, char const * const no_color_label,
+GtkWidget *color_combo_new      (char **icon, const char *no_color_label,
 				 GdkColor *default_color, gchar *group_name);
 
 END_GNOME_DECLS
